@@ -25,7 +25,7 @@ bool ExampleApplet::load()
 {
     DCORE_USE_NAMESPACE;
     std::unique_ptr<DConfig> config(DConfig::create("org.deepin.dde.shell", "org.deepin.ds.example"));
-    return config->value("loadAppletExampleData").toBool();
+    return config->value("loadAppletExampleData", true).toBool();
 }
 
 bool ExampleApplet::init()
@@ -56,6 +56,11 @@ void ExampleApplet::setUserData(bool newUserData)
     emit userDataChanged();
 }
 
+QString ExampleApplet::call(const QString &id)
+{
+    return id + QString("-done");
+}
+
 D_APPLET_CLASS(ExampleApplet)
 
 #include "exampleapplet.moc"
