@@ -15,6 +15,7 @@ QT_END_NAMESPACE
 
 DS_BEGIN_NAMESPACE
 class DApplet;
+class DAppletProxy;
 
 class DQmlGlobalPrivate;
 class DQmlGlobal : public QObject, public DTK_CORE_NAMESPACE::DObject
@@ -26,8 +27,8 @@ public:
     explicit DQmlGlobal(QObject *parent = nullptr);
     ~DQmlGlobal() override;
 
-    Q_INVOKABLE DApplet *applet(const QString &pluginId) const;
-    Q_INVOKABLE QList<DApplet *> appletList(const QString &pluginId) const;
+    Q_INVOKABLE DAppletProxy *applet(const QString &pluginId) const;
+    Q_INVOKABLE QList<DAppletProxy *> appletList(const QString &pluginId) const;
     Q_INVOKABLE static void closeChildrenWindows(QWindow *target);
     Q_INVOKABLE bool grabKeyboard(QWindow *target, bool grab = true);
     Q_INVOKABLE bool grabMouse(QWindow *target, bool grab = true);
