@@ -2,15 +2,14 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-#include "panel.h"
-#include "constants.h"
 #include "dockpanel.h"
+#include "constants.h"
 #include "dockadaptor.h"
-#include "environments.h"
 #include "docksettings.h"
+#include "panel.h"
 #include "pluginfactory.h"
-#include "x11dockhelper.h"
 #include "waylanddockhelper.h"
+#include "x11dockhelper.h"
 
 // for old api compatible
 #include "dockdbusproxy.h"
@@ -31,14 +30,15 @@ Q_LOGGING_CATEGORY(dockLog, "dde.shell.dock")
 
 namespace dock {
 
-DockPanel::DockPanel(QObject * parent)
+DockPanel::DockPanel(QObject *parent)
     : DPanel(parent)
     , m_theme(ColorTheme::Dark)
-    , m_hideState(Hide)
+    , m_hideState(Show)
     , m_dockScreen(nullptr)
     , m_loadTrayPlugins(new LoadTrayPlugins(this))
     , m_compositorReady(false)
     , m_launcherShown(false)
+    , m_contextDragging(false)
 {
     connect(this, &DockPanel::compositorReadyChanged, this, [this] {
         if (!m_compositorReady) return;
@@ -74,8 +74,11 @@ bool DockPanel::init()
                 if (m_dockScreen != window()->screen() && qApp->screens().contains( m_dockScreen)) {
                     qWarning() << "m_dockScreen" << m_dockScreen << m_dockScreen->name() << "window()->screen()" << window()->screen() << window()->screen()->name();
                     QTimer::singleShot(10, this, [this](){
-                        window()->setScreen(m_dockScreen); 
+                        window()->setScreen(m_dockScreen);
+                        onWindowGeometryChanged();
                     });
+                } else {
+                    onWindowGeometryChanged();
                 }
             }else {
                 m_dockScreen = window()->screen();
@@ -85,8 +88,6 @@ bool DockPanel::init()
     connect(this, &DockPanel::hideModeChanged, this, [this](){
         if (hideMode() == KeepShowing)
             setHideState(Show);
-        if (hideMode() == KeepHidden)
-            setHideState(Hide);
     });
     connect(SETTINGS, &DockSettings::positionChanged, this, [this, dockDaemonAdaptor](){
         Q_EMIT positionChanged(position());
@@ -100,6 +101,7 @@ bool DockPanel::init()
     connect(SETTINGS, &DockSettings::showInPrimaryChanged, this, [this, dockDaemonAdaptor](){
         updateDockScreen();
         Q_EMIT dockDaemonAdaptor->FrontendWindowRectChanged(frontendWindowRect());
+        Q_EMIT showInPrimaryChanged(showInPrimary());
     });
 
     connect(this, &DockPanel::frontendWindowRectChanged, dockDaemonAdaptor, &DockDaemonAdaptor::FrontendWindowRectChanged);
@@ -125,7 +127,7 @@ bool DockPanel::init()
     QObject::connect(this, &DApplet::rootObjectChanged, this, [this]() {
         if (rootObject()) {
             // those connections need connect after DPanel::init() which create QQuickWindow
-            // xChanged yChanged not woker on wayland, so use above positionChanged instead
+            // xChanged yChanged not worked on wayland, so use above positionChanged instead
             // connect(window(), &QQuickWindow::xChanged, this, &DockPanel::onWindowGeometryChanged);
             // connect(window(), &QQuickWindow::yChanged, this, &DockPanel::onWindowGeometryChanged);
             connect(window(), &QQuickWindow::widthChanged, this, &DockPanel::onWindowGeometryChanged);
@@ -133,6 +135,11 @@ bool DockPanel::init()
             QMetaObject::invokeMethod(this, &DockPanel::onWindowGeometryChanged);
             if (showInPrimary())
                 updateDockScreen();
+            else {
+                m_dockScreen = window()->screen();
+            }
+            rootObject()->installEventFilter(this);
+            Q_EMIT devicePixelRatioChanged(window()->devicePixelRatio());
         }
     });
 
@@ -149,17 +156,7 @@ bool DockPanel::init()
         m_helper = new X11DockHelper(this);
     }
 
-
-    connect(m_helper, &DockHelper::hideStateChanged, this, [this](){
-        if (hideMode() == KeepShowing || m_launcherShown) return;
-        m_hideState = m_helper->hideState();
-        Q_EMIT hideStateChanged(m_hideState);
-    });
-
     QMetaObject::invokeMethod(this, [this, dockDaemonAdaptor]() {
-        m_hideState = hideMode() == KeepShowing ? Show : m_helper->hideState();
-        Q_EMIT hideStateChanged(m_hideState);
-
         Q_EMIT dockDaemonAdaptor->FrontendWindowRectChanged(frontendWindowRect());
     });
 
@@ -297,6 +294,10 @@ void DockPanel::setCompositorReady(bool ready)
 
 HideState DockPanel::hideState()
 {
+    if (m_launcherShown) {
+        return Show;
+    }
+
     return m_hideState;
 }
 
@@ -326,7 +327,7 @@ void DockPanel::openDockSettings() const
         .path(QStringLiteral("/org/deepin/dde/ControlCenter1"))
         .interface(QStringLiteral("org.deepin.dde.ControlCenter1"))
         .method(QStringLiteral("ShowPage"))
-        .arg(QStringLiteral("personalization/desktop/dock"))
+        .arg(QStringLiteral("personalization/dock"))
         .call();
 }
 
@@ -334,19 +335,16 @@ void DockPanel::launcherVisibleChanged(bool visible)
 {
     if (visible == m_launcherShown) return;
 
+    const HideState oldHideState = hideState();
     m_launcherShown = visible;
-    if (m_launcherShown) {
-        setHideState(Show);
-    } else {
-        if (hideMode() != KeepShowing) {
-            setHideState(m_helper->hideState());
-        } else {
-            setHideState(Show);
-        }
+    const HideState newHideState = hideState();
+
+    if (newHideState != oldHideState) {
+        Q_EMIT hideStateChanged(newHideState);
     }
 }
 
-void DockPanel::updateDockScreen() 
+void DockPanel::updateDockScreen()
 {
     auto win = window();
     if (!win)
@@ -368,7 +366,6 @@ void DockPanel::setShowInPrimary(bool newShowInPrimary)
         connect(qApp, &QGuiApplication::primaryScreenChanged, this, &DockPanel::updateDockScreen, Qt::UniqueConnection);
     else
         disconnect(qApp, &QGuiApplication::primaryScreenChanged, this, &DockPanel::updateDockScreen);
-    Q_EMIT showInPrimaryChanged(showInPrimary());
 }
 
 D_APPLET_CLASS(DockPanel)
@@ -393,6 +390,45 @@ void DockPanel::setDockScreen(QScreen *screen)
     m_dockScreen = screen;
     window()->setScreen(m_dockScreen);
     Q_EMIT dockScreenChanged(m_dockScreen);
+    Q_EMIT screenNameChanged();
+}
+
+QString DockPanel::screenName() const
+{
+    if (!m_dockScreen)
+        return {};
+    return m_dockScreen->name();
+}
+
+qreal DockPanel::devicePixelRatio() const
+{
+    if (!window())
+        return 1.0;
+    return window()->devicePixelRatio();
+}
+
+bool DockPanel::eventFilter(QObject *watched, QEvent *event)
+{
+    if (watched == window() && event->type() == QEvent::DevicePixelRatioChange) {
+        Q_EMIT devicePixelRatioChanged(window()->devicePixelRatio());
+    }
+
+    return false;
+}
+
+bool DockPanel::contextDragging() const
+{
+    return m_contextDragging;
+}
+
+void DockPanel::setContextDragging(bool newContextDragging)
+{
+    if (m_contextDragging == newContextDragging)
+        return;
+    m_contextDragging = newContextDragging;
+    if (!m_contextDragging)
+        m_helper->checkNeedHideOrNot();
+    emit contextDraggingChanged();
 }
 }
 
