@@ -31,9 +31,14 @@ class DockPanel : public DS_NAMESPACE::DPanel, public QDBusContext
     Q_PROPERTY(ItemAlignment itemAlignment READ itemAlignment WRITE setItemAlignment NOTIFY itemAlignmentChanged FINAL)
     Q_PROPERTY(IndicatorStyle indicatorStyle READ indicatorStyle WRITE setIndicatorStyle NOTIFY indicatorStyleChanged FINAL)
     Q_PROPERTY(bool showInPrimary READ showInPrimary WRITE setShowInPrimary NOTIFY showInPrimaryChanged FINAL)
+    Q_PROPERTY(QString screenName READ screenName NOTIFY screenNameChanged FINAL)
+
+    Q_PROPERTY(qreal devicePixelRatio READ devicePixelRatio NOTIFY devicePixelRatioChanged FINAL)
 
     Q_PROPERTY(bool debugMode READ debugMode FINAL CONSTANT)
 
+    Q_PROPERTY(bool contextDragging READ contextDragging WRITE setContextDragging NOTIFY contextDraggingChanged FINAL)
+
 public:
     explicit DockPanel(QObject *parent = nullptr);
 
@@ -79,6 +84,15 @@ public:
     void setHideState(HideState newHideState);
     QScreen* dockScreen();
     void setDockScreen(QScreen *screen);
+    QString screenName() const;
+
+    qreal devicePixelRatio() const;
+
+    bool contextDragging() const;
+    void setContextDragging(bool newContextDragging);
+
+protected:
+    bool eventFilter(QObject *watched, QEvent *event) override;
 
 private Q_SLOTS:
     void onWindowGeometryChanged();
@@ -99,7 +113,11 @@ Q_SIGNALS:
     void indicatorStyleChanged(IndicatorStyle style);
     void showInPrimaryChanged(bool showInPrimary);
     void dockScreenChanged(QScreen *screen);
+    void screenNameChanged();
     void requestClosePopup();
+    void devicePixelRatioChanged(qreal ratio);
+
+    void contextDraggingChanged();
 
 private:
     ColorTheme m_theme;
@@ -109,6 +127,7 @@ private:
     LoadTrayPlugins *m_loadTrayPlugins;
     bool m_compositorReady;
     bool m_launcherShown;
+    bool m_contextDragging;
 };
 
 }
