@@ -4,8 +4,6 @@
 
 #pragma once
 
-#include "dsglobal.h"
-
 #include <QWindow>
 #include <QString>
 #include <QObject>
@@ -26,8 +24,17 @@ class AbstractWindow : public QObject
     Q_PROPERTY(bool shouldSkip READ shouldSkip NOTIFY shouldSkipChanged FINAL)
 
 public:
-    enum Roles { winIdRole = Qt::UserRole + 1, pidRole, identityRole, winIconRole, winTitleRole, activeRole, shouldSkipRole };
+    enum Roles {
+        winIdRole = Qt::UserRole + 1,
+        pidRole,
+        identityRole,
+        winIconRole,
+        winTitleRole,
+        activieRole,
+        shouldSkipRole
+    };
     Q_ENUM(Roles)
+
     virtual ~AbstractWindow() {};
 
     virtual uint32_t id() = 0;
