@@ -2,7 +2,6 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-#include "dsglobal.h"
 #include "itemmodel.h"
 #include "abstractitem.h"
 #include "taskmanagersettings.h"
@@ -75,9 +74,12 @@ QVariant ItemModel::data(const QModelIndex &index, int role) const
 
 void ItemModel::moveTo(const QString &id, int dIndex)
 {
+    // simply do nothing if dIndex is invalid
+    if (dIndex >= m_items.size() || dIndex < 0) {
+        return;
+    }
+
     auto sItem = getItemById(id);
-    // FIXME: Somehow, the dIndex value can go beyond m_item.size().
-    dIndex = qMin(dIndex, m_items.size() - 1);
     auto dItem = m_items.at(dIndex);
 
     int sIndex = m_items.indexOf(sItem);
