@@ -12,7 +12,6 @@
 #include "globals.h"
 #include "itemmodel.h"
 #include "pluginfactory.h"
-#include "pluginloader.h"
 #include "rolecombinemodel.h"
 #include "taskmanager.h"
 #include "taskmanageradaptor.h"
@@ -22,11 +21,13 @@
 #include <QGuiApplication>
 #include <QStringLiteral>
 
+#include <appletbridge.h>
+
 #ifdef BUILD_WITH_X11
 #include "x11windowmonitor.h"
 #endif
 
-Q_LOGGING_CATEGORY(taskManagerLog, "dde.shell.dock.taskmanager", QtInfoMsg)
+Q_LOGGING_CATEGORY(taskManagerLog, "dde.shell.dock.taskmanager", QtDebugMsg)
 
 #define Settings TaskManagerSettings::instance()
 
@@ -39,6 +40,7 @@ namespace dock {
 
 TaskManager::TaskManager(QObject* parent)
     : DContainment(parent)
+    , m_windowFullscreen(false)
 {
     qRegisterMetaType<ObjectInterfaceMap>();
     qDBusRegisterMetaType<ObjectInterfaceMap>();
@@ -80,28 +82,10 @@ bool TaskManager::init()
 
     DApplet::init();
 
-    auto findApplet = [this](QString pluginId) {
-        QList<DS_NAMESPACE::DApplet *> ret;
-        auto root = qobject_cast<DS_NAMESPACE::DContainment *>(DS_NAMESPACE::DPluginLoader::instance()->rootApplet());
-
-        QQueue<DS_NAMESPACE::DContainment *> containments;
-        containments.enqueue(root);
-        while (!containments.isEmpty()) {
-            DS_NAMESPACE::DContainment *containment = containments.dequeue();
-            for (const auto applet : containment->applets()) {
-                if (auto item = qobject_cast<DS_NAMESPACE::DContainment *>(applet)) {
-                    containments.enqueue(item);
-                }
-                if (applet->pluginId() == pluginId)
-                    ret << applet;
-            }
-        }
-        return ret;
-    };
-
-    auto c = findApplet("org.deepin.ds.dde-apps");
-    if (c.size() > 0) {
-        auto model = c.first()->property("appModel").value<QAbstractItemModel *>();
+    DS_NAMESPACE::DAppletBridge bridge("org.deepin.ds.dde-apps");
+    if (auto applet = bridge.applet()) {
+        auto model = applet->property("appModel").value<QAbstractItemModel *>();
+        Q_ASSERT(model);
         m_activeAppModel =
             new RoleCombineModel(m_windowMonitor.data(), model, AbstractWindow::identityRole, [](QVariant data, QAbstractItemModel *model) -> QModelIndex {
                 auto roleNames = model->roleNames();
@@ -109,12 +93,14 @@ bool TaskManager::init()
 
                 auto indentifies = data.toStringList();
                 for (auto id : indentifies) {
-                    if (id.isEmpty())
+                    if (id.isEmpty()) {
                         continue;
+                    }
 
                     for (auto identifiedOrder : identifiedOrders) {
                         auto res = model->match(model->index(0, 0), roleNames.key(identifiedOrder), id, 1, Qt::MatchFixedString | Qt::MatchWrap);
                         if (res.size() > 0 && res.first().isValid()) {
+                            qCDebug(taskManagerLog()) << "Window match id:" << id << "role:" << identifiedOrder << "data:" << res.first().data() << "desktop id:" << res.first().data(roleNames.key("desktopId"));
                             return res.first();
                         }
                     }
@@ -124,9 +110,12 @@ bool TaskManager::init()
             });
     }
 
-    QTimer::singleShot(500, [this]() {
-        if (m_windowMonitor)
-            m_windowMonitor->start();
+    if (m_windowMonitor)
+        m_windowMonitor->start();
+
+    connect(m_windowMonitor.data(), &AbstractWindowMonitor::windowFullscreenChanged, this, [this] (bool isFullscreen) {
+        m_windowFullscreen = isFullscreen;
+        emit windowFullscreenChanged(isFullscreen);
     });
     return true;
 }
@@ -143,7 +132,6 @@ void TaskManager::handleWindowAdded(QPointer<AbstractWindow> window)
     // TODO: remove below code and use use model replaced.
     QModelIndexList res;
     if (m_activeAppModel) {
-        qDebug() << m_activeAppModel->rowCount();
         res = m_activeAppModel->match(m_activeAppModel->index(0, 0), AbstractWindow::winIdRole, window->id());
     }
 
@@ -151,14 +139,17 @@ void TaskManager::handleWindowAdded(QPointer<AbstractWindow> window)
     QString desktopId;
     if (res.size() > 0) {
         desktopId = res.first().data(m_activeAppModel->roleNames().key("desktopId")).toString();
+        qCDebug(taskManagerLog()) << "identify bt model:" << desktopId;
     }
 
     if (!desktopId.isEmpty()) {
         desktopfile = DESKTOPFILEFACTORY::createById(desktopId, "amAPP");
+        qCDebug(taskManagerLog()) <<  "identify bt AM:" << desktopId;
     }
 
     if (desktopfile.isNull() || !desktopfile->isValied().first) {
         desktopfile = DESKTOPFILEFACTORY::createByWindow(window);
+        qCDebug(taskManagerLog()) <<  "identify bt Fallback:" << desktopId;
     }
 
     auto appitem = desktopfile->getAppItem();
@@ -193,6 +184,14 @@ void TaskManager::clickItem(const QString& itemId, const QString& menuId)
     item->handleClick(menuId);
 }
 
+void TaskManager::dropFilesOnItem(const QString& itemId, const QStringList& urls)
+{
+    auto item = ItemModel::instance()->getItemById(itemId);
+    if(!item) return;
+
+    item->handleFileDrop(urls);
+}
+
 void TaskManager::showItemPreview(const QString &itemId, QObject* relativePositionItem, int32_t previewXoffset, int32_t previewYoffset, uint32_t direction)
 {
     auto item = ItemModel::instance()->getItemById(itemId).get();
@@ -238,7 +237,7 @@ void TaskManager::loadDockedAppItems()
         if (appitem.isNull()) {
             appitem = new AppItem(appid);
         }
-    
+
         appitem->setDesktopFileParser(desktopfile);
         ItemModel::instance()->addItem(appitem);
     }
@@ -254,10 +253,16 @@ QString TaskManager::desktopIdToAppId(const QString& desktopId)
     return Q_LIKELY(desktopId.endsWith(".desktop")) ? desktopId.chopped(8) : desktopId;
 }
 
-bool TaskManager::requestDockByDesktopId(const QString& appID)
+bool TaskManager::requestDockByDesktopId(const QString& desktopID)
 {
-    if (appID.startsWith("internal/")) return false;
-    return RequestDock(desktopIdToAppId(appID));
+    if (desktopID.startsWith("internal/")) return false;
+    return RequestDock(desktopIdToAppId(desktopID));
+}
+
+bool TaskManager::requestUndockByDesktopId(const QString& desktopID)
+{
+    if (desktopID.startsWith("internal/")) return false;
+    return RequestUndock(desktopIdToAppId(desktopID));
 }
 
 bool TaskManager::RequestDock(QString appID)
@@ -318,6 +323,11 @@ bool TaskManager::windowSplit()
     return Settings->isWindowSplit();
 }
 
+bool TaskManager::windowFullscreen()
+{
+    return m_windowFullscreen;
+}
+
 D_APPLET_CLASS(TaskManager)
 }
 
