@@ -21,6 +21,7 @@ class TaskManager : public DS_NAMESPACE::DContainment
     Q_PROPERTY(ItemModel* dataModel READ dataModel NOTIFY dataModelChanged)
 
     Q_PROPERTY(bool windowSplit READ windowSplit NOTIFY windowSplitChanged)
+    Q_PROPERTY(bool windowFullscreen READ windowFullscreen NOTIFY windowFullscreenChanged)
     Q_PROPERTY(bool allowForceQuit READ allowForceQuit NOTIFY allowedForceQuitChanged)
 
 public:
@@ -32,15 +33,18 @@ public:
     virtual bool load() override;
 
     bool windowSplit();
+    bool windowFullscreen();
     bool allowForceQuit();
 
     Q_INVOKABLE QString desktopIdToAppId(const QString& desktopId);
-    Q_INVOKABLE bool requestDockByDesktopId(const QString& appID);
+    Q_INVOKABLE bool requestDockByDesktopId(const QString& desktopID);
+    Q_INVOKABLE bool requestUndockByDesktopId(const QString& desktopID);
     Q_INVOKABLE bool RequestDock(QString appID);
     Q_INVOKABLE bool IsDocked(QString appID);
     Q_INVOKABLE bool RequestUndock(QString appID);
 
     Q_INVOKABLE void clickItem(const QString& itemid, const QString& menuId);
+    Q_INVOKABLE void dropFilesOnItem(const QString& itemId, const QStringList& urls);
     Q_INVOKABLE void showItemPreview(const QString& itemId, QObject* relativePositionItem, int32_t previewXoffset, int32_t previewYoffset, uint32_t direction);
     Q_INVOKABLE void hideItemPreview();
 
@@ -49,6 +53,7 @@ public:
 Q_SIGNALS:
     void dataModelChanged();
     void windowSplitChanged();
+    void windowFullscreenChanged(bool);
     void allowedForceQuitChanged();
 
 private Q_SLOTS:
@@ -60,6 +65,7 @@ private:
 private:
     QScopedPointer<AbstractWindowMonitor> m_windowMonitor;
     RoleCombineModel *m_activeAppModel = nullptr;
+    bool m_windowFullscreen;
 };
 
 }
