@@ -11,22 +11,23 @@
 #include <cstdint>
 #include <unistd.h>
 
-#include <QFile>
-#include <QTimer>
-#include <QEvent>
-#include <QWindow>
-#include <QPixmap>
-#include <QLayout>
-#include <QScreen>
-#include <QPainter>
-#include <QByteArray>
-#include <QDBusReply>
-#include <QMouseEvent>
-#include <QDBusInterface>
-#include <QLoggingCategory>
-#include <QDBusUnixFileDescriptor>
-#include <QPainterPath>
 #include <DIconButton>
+#include <QByteArray>
+#include <QDBusInterface>
+#include <QDBusReply>
+#include <QDBusUnixFileDescriptor>
+#include <QEvent>
+#include <QFile>
+#include <QLayout>
+#include <QLoggingCategory>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QPainterPath>
+#include <QPixmap>
+#include <QScreen>
+#include <QTimer>
+#include <QWindow>
+#include <QtConcurrent>
 
 #include <DStyle>
 #include <DPlatformHandle>
@@ -161,9 +162,13 @@ private:
     {
         m_previewPixmaps.clear();
         if (!m_item.isNull()) {
-            for (const auto &window : m_item->getAppendWindows()) {
+            const auto previewPixmaps = QtConcurrent::blockingMapped(m_item->getAppendWindows(), [this](const QPointer<AbstractWindow> &window) {
                 auto previewPixmap = fetchWindowPreview(window->id());
-                m_previewPixmaps.insert(window->id(), previewPixmap);
+                return QPair<uint32_t, QPixmap>{window->id(), previewPixmap};
+            });
+
+            for (const auto &item : previewPixmaps) {
+                m_previewPixmaps.insert(item.first, item.second);
             }
         }
     }
@@ -259,14 +264,13 @@ public:
         QRect hoverRect = option.rect;
 
         QPen pen;
-        if (WM_HELPER->hasComposite()) {
+        if (WM_HELPER->hasComposite() && WM_HELPER->hasBlurWindow()) {
             auto pixmap = index.data(WindowPreviewContentRole).value<QPixmap>();
-            auto size = calSize(pixmap.size());
-            auto scaledPixmap = pixmap.scaled(size, Qt::KeepAspectRatio, Qt::FastTransformation);
-            scaledPixmap.setDevicePixelRatio(qApp->devicePixelRatio());
+            auto size = calSize(pixmap.size()); 
 
             DStyleHelper dstyle(m_listView->style());
             const int radius = dstyle.pixelMetric(DStyle::PM_FrameRadius);
+            painter->setRenderHint(QPainter::Antialiasing);
 
             painter->save();
             painter->setPen(Qt::NoPen);
@@ -280,19 +284,18 @@ public:
 
 
             painter->save();
-            painter->setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
             pen.setWidth(1);
             pen.setColor(themeType == DGuiApplicationHelper::DarkType ? QColor(255, 255, 255, 255 * 0.1) : QColor(0, 0, 0, 255 * 0.1));
             painter->setPen(pen);
             QRect imageRect(
-                (option.rect.left() + ((option.rect.width() - scaledPixmap.width()) / 2)),
-                (option.rect.top() + ((option.rect.height() - scaledPixmap.height()) / 2)),
-                scaledPixmap.width(),
-                scaledPixmap.height());
+                (option.rect.left() + ((option.rect.width() - size.width()) / 2)),
+                (option.rect.top() + ((option.rect.height() - size.height()) / 2)),
+                size.width(),
+                size.height());
             QPainterPath clipPath;
             clipPath.addRoundedRect(imageRect, radius, radius);
             painter->setClipPath(clipPath);
-            painter->drawPixmap(imageRect, scaledPixmap);
+            painter->drawPixmap(imageRect, pixmap);
             painter->setClipping(false);
             painter->drawRoundedRect(imageRect, radius, radius);
             if (option.state.testFlag(QStyle::State_MouseOver)) {
@@ -334,7 +337,7 @@ public:
 
     virtual QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
-        if (!WM_HELPER->hasComposite()) {
+        if (!WM_HELPER->hasComposite() || !WM_HELPER->hasBlurWindow()) {
             return QSize(PREVIEW_CONTENT_MAX_WIDTH + PREVIEW_HOVER_BORDER * 2, PREVIEW_TITLE_HEIGHT + PREVIEW_HOVER_BORDER * 2);
         }
 
@@ -391,12 +394,6 @@ private:
 
 };
 
-const QString AppearanceService = QStringLiteral("org.deepin.dde.Appearance1");
-const QString AppearancePath = QStringLiteral("/org/deepin/dde/Appearance1");
-const QString PropertiesInterface = QStringLiteral("org.freedesktop.DBus.Properties");
-const QString PropertiesChanged = QStringLiteral("PropertiesChanged");
-const QString OpacityStr = QStringLiteral("Opacity");
-
 X11WindowPreviewContainer::X11WindowPreviewContainer(X11WindowMonitor* monitor, QWidget *parent)
     : DBlurEffectWidget(parent)
     , m_direction(0)
@@ -413,8 +410,6 @@ X11WindowPreviewContainer::X11WindowPreviewContainer(X11WindowMonitor* monitor,
     setMouseTracking(true);
     initUI();
 
-    QDBusConnection::sessionBus().connect(AppearanceService, AppearancePath, PropertiesInterface, PropertiesChanged, this, SLOT(onPropertiesChanged(QDBusMessage)));
-
     connect(m_hideTimer, &QTimer::timeout, this, &X11WindowPreviewContainer::callHide);
 
     connect(m_closeAllButton, &DIconButton::clicked, this, [this](){
@@ -436,17 +431,6 @@ X11WindowPreviewContainer::X11WindowPreviewContainer(X11WindowMonitor* monitor,
 
 }
 
-void X11WindowPreviewContainer::onPropertiesChanged(const QDBusMessage &message)
-{
-    QVariantMap changedProps = qdbus_cast<QVariantMap>(message.arguments().at(1).value<QDBusArgument>());
-    for (QVariantMap::const_iterator it = changedProps.cbegin(); it != changedProps.cend(); ++it) {
-        if (it.key().toLatin1() == OpacityStr) {
-            setMaskAlpha(it.value().toFloat() * 255);
-            return;
-        }
-    }
-}
-
 void X11WindowPreviewContainer::showPreview(const QPointer<AppItem> &item, const QPointer<QWindow> &window, int32_t previewXoffset, int32_t previewYoffset, uint32_t direction)
 {
     if (!m_previewItem.isNull()) {
@@ -477,7 +461,7 @@ void X11WindowPreviewContainer::showPreview(const QPointer<AppItem> &item, const
 void X11WindowPreviewContainer::updateOrientation()
 {
 
-    if (m_direction % 2 == 0 && WM_HELPER->hasComposite()) {
+    if (m_direction % 2 == 0 && WM_HELPER->hasComposite() && WM_HELPER->hasBlurWindow()) {
         m_view->setFlow(QListView::LeftToRight);
     } else {
         m_view->setFlow(QListView::TopToBottom);
