@@ -17,7 +17,6 @@
 #include <DBlurEffectWidget>
 #include <DGuiApplicationHelper>
 #include <DIconButton>
-#include <QDBusMessage>
 
 DWIDGET_USE_NAMESPACE
 
@@ -90,7 +89,6 @@ public Q_SLOTS:
 private Q_SLOTS:
     void updateOrientation();
     void callHide();
-    void onPropertiesChanged(const QDBusMessage &message);
 
 private:
     bool m_isPreviewEntered;
