@@ -4,7 +4,6 @@
 <context>
     <name>ActionShowStashDelegate</name>
     <message>
-        <location filename="../package/ActionShowStashDelegate.qml" line="95"/>
         <source>Application tray</source>
         <translation>应用托盘</translation>
     </message>
@@ -12,7 +11,6 @@
 <context>
     <name>ActionToggleCollapseDelegate</name>
     <message>
-        <location filename="../package/ActionToggleCollapseDelegate.qml" line="30"/>
         <source>Collapse tray</source>
         <translation>折叠托盘</translation>
     </message>
@@ -20,7 +18,6 @@
 <context>
     <name>PanelTrayItem</name>
     <message>
-        <location filename="../quickpanel/PanelTrayItem.qml" line="30"/>
         <source>Quick actions</source>
         <translation>快捷设置</translation>
     </message>
