@@ -206,6 +206,11 @@ void TraySortOrderModel::setSurfaceVisible(const QString &surfaceId, bool visibl
     updateVisualIndexes();
 }
 
+bool TraySortOrderModel::isDisplayedSurface(const QString &surfaceId) const
+{
+    return !m_hiddenIds.contains(surfaceId);
+}
+
 QStandardItem *TraySortOrderModel::findItemByVisualIndex(int visualIndex, VisualSections visualSection) const
 {
     QStandardItem * result = nullptr;
@@ -324,8 +329,7 @@ QStandardItem *TraySortOrderModel::createTrayItem(const QString &name,
 void TraySortOrderModel::updateVisualIndexes()
 {
     for (int i = 0; i < rowCount(); i++) {
-        auto results = item(i);
-        results->setData(-1, TraySortOrderModel::VisualIndexRole);
+        item(i)->setData(-1, TraySortOrderModel::VisualIndexRole);
     }
 
     // stashed action
